package com.m2pool.chat.interceptor;


import com.m2pool.chat.config.CustomWebSocketConfig;
import com.m2pool.chat.entity.StompPrincipal;
import lombok.Data;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.lang.Nullable;
import org.springframework.messaging.Message;
import org.springframework.messaging.MessageChannel;
import org.springframework.messaging.MessageDeliveryException;
import org.springframework.messaging.simp.SimpMessageHeaderAccessor;
import org.springframework.messaging.simp.stomp.StompCommand;
import org.springframework.messaging.simp.stomp.StompHeaderAccessor;
import org.springframework.messaging.support.ChannelInterceptor;
import org.springframework.messaging.support.MessageHeaderAccessor;

import java.util.ArrayList;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;

import static com.m2pool.chat.constant.CustomHeader.*;
import static com.m2pool.chat.constant.UserType.CUSTOMER;


/**
 * @ClassName WebsocketChannelInterceptor
 * @Description websocket channel 通道拦截器   适用于前端@stomp/stompjs 实现.
 * @Author yyb
 * @Date 2025/4/10 15:44
 */
@Data
public class WebsocketChannelInterceptor implements ChannelInterceptor {
    private static final Logger LOGGER = LoggerFactory.getLogger(WebsocketChannelInterceptor.class);

    /**
     * 当前加入链接的ip
     */
    private static final ConcurrentHashMap<String, Boolean> ipConnectionCountMap = new ConcurrentHashMap<>();


    /**
     * 当前链接数
     */
    private static final AtomicInteger connectionCount = new AtomicInteger(0);

    /**
     * websocket 一些配置
     */
    private CustomWebSocketConfig customWebSocketConfig;

    public WebsocketChannelInterceptor(CustomWebSocketConfig customWebSocketConfig) {
        this.customWebSocketConfig = customWebSocketConfig;
    }

    /**
     * websocket channel 通道拦截器
     * @param message
     * @param channel
     * @return
     */
    @Override
    public Message<?> preSend(Message<?> message, MessageChannel channel) {
        //获取链接建立时的请求头信息
        StompHeaderAccessor accessor = StompHeaderAccessor.wrap(message);
        if (accessor.getCommand() == StompCommand.CONNECT ) {
            StompHeaderAccessor mha = MessageHeaderAccessor.getAccessor(message, StompHeaderAccessor.class);
            Object raw = message.getHeaders().get(SimpMessageHeaderAccessor.NATIVE_HEADERS);
            int type = Integer.parseInt(getConnectCustomHeaders(raw, TYPE));
            String email = getConnectCustomHeaders(raw, EMAIL);

            //最大链接数限制
            maxConnectionsLimit();
            //根据客服端ip + 用户类型限制连接数
            ipLimit(accessor,type);

            if(mha == null){
                throw new MessageDeliveryException("get MessageHeaderAccessor failed,can not set Principal,connect failed");
            }
            //链接请求头中用户信息存入stomp中
            mha.setUser(new StompPrincipal(email,type));
        }
        if (accessor.getCommand() == StompCommand.SEND) {
            LOGGER.info("------------websocket send message");
        }

        if (accessor.getCommand() == StompCommand.SUBSCRIBE) {
            LOGGER.info("------------websocket subscribe message");
        }

        if (accessor.getCommand() == StompCommand.UNSUBSCRIBE) {
            LOGGER.info("------------websocket unsubscribe");
        }

        if (accessor.getCommand() == StompCommand.DISCONNECT){
            disconnectHandler(accessor);
            LOGGER.info("------------websocket disconnect");
        }
        return message;
    }

    /**
     * 最大连接数限制
     */
    private void maxConnectionsLimit(){
        if(connectionCount.incrementAndGet() >= customWebSocketConfig.getMaxConnections()){
            LOGGER.info("reach max connections,refuse connect");
            throw new MessageDeliveryException("reach max connections,refuse connect");
        }
    }

    /**
     * ip 链接个数限制
     * @param accessor
     */
    private void ipLimit(StompHeaderAccessor accessor,int type){
        Map<String, Object> sessionAttributes = accessor.getSessionAttributes();
        if (sessionAttributes != null) {
            String ipAddr = (String) sessionAttributes.get(IPADDR);
            if(type != CUSTOMER && ipConnectionCountMap.get(ipAddr) != null){
                throw new MessageDeliveryException("this ip maximum number of connections is reached");
            }
            ipConnectionCountMap.put(ipAddr,true);
        }else{
            throw new MessageDeliveryException("not find email and type headers");
        }
    }

    /**
     * 获取stomp自定义请求头
     * @param raw
     * @param headerKey
     * @return
     */
    private String getConnectCustomHeaders(Object raw,String headerKey){
        String headerValue = "";
        if (raw instanceof Map) {
            Object value = ((Map<?, ?>) raw).get(headerKey);
            if(value instanceof ArrayList){
                headerValue = ((ArrayList<?>) value).get(0).toString();
            }
        }
        return headerValue;
    }

    /**
     * 断开链接处理
     */
    private void disconnectHandler(StompHeaderAccessor accessor){
        //连接数减一
        connectionCount.decrementAndGet();
        //断开链接的ip
        Map<String, Object> sessionAttributes = accessor.getSessionAttributes();
        if (sessionAttributes != null) {
            String ipAddr = (String) sessionAttributes.get(IPADDR);
            if(ipConnectionCountMap.get(ipAddr) != null){
                ipConnectionCountMap.remove(ipAddr);
            }
        }else{
            throw new MessageDeliveryException("not find email and type headers");
        }
    }


    @Override
    public void postSend(Message<?> message, MessageChannel channel, boolean sent) {
        LOGGER.info("------------WebsocketChannelInterceptor-postSend");
    }

    @Override
    public void afterSendCompletion(Message<?> message, MessageChannel channel, boolean sent, @Nullable Exception ex) {
        LOGGER.info("-----------WebsocketChannelInterceptor-afterSendCompletion");
    }

    @Override
    public boolean preReceive(MessageChannel channel) {
        LOGGER.info("----------WebsocketChannelInterceptor-preReceive");
        return true;
    }

    @Override
    public Message<?> postReceive(Message<?> message, MessageChannel channel) {
        LOGGER.info("----------WebsocketChannelInterceptor-postReceive");
        return message;
    }

    @Override
    public void afterReceiveCompletion(@Nullable Message<?> message, MessageChannel channel, @Nullable Exception ex) {
        LOGGER.info("----------WebsocketChannelInterceptor-afterReceiveCompletion");
    }
}
